'use strict';

const sUtil = require('../lib/util');
const mathoid = require('../lib/math');
const emitError = mathoid.emitError;
const BBPromise = require('bluebird');
const zlib = BBPromise.promisifyAll(require('zlib'));

/**
 * The main router object
 */
const router = sUtil.router();

/**
 * The main application object reported when this module is require()d
 */
let app;

/**
 * GET /
 * Performs the check get request
 */
router.get('/get/:outformat?/:type?/:q?', (req, res) => {
    if (!(req.params.q)) {
        emitError('q (query) parameter is missing!');
    }
    res.set('Warning',
        '299 mathoid Rendering via GET request discouraged. Consider using the POST endpoint.');

    //begin CJK spacing - by libattery
    var cjk = new RegExp(
    "(["
    +"\u1100-\u11FF\u3130-\u318F\uAC00-\uD7AF" //Korean
    +"\u2E80-\u2EFF\u31C0-\u31EF\u3200-\u32FF" //Chinese
    +"\u3400-\u4DBF\u4E00-\u9FBF\uF900-\uFAFF" //Chinese
    +"\u3000-\u303f\u3040-\u30FF\u31F0-\u31FF\uff00-\uff9f" //Japanese
    +"])(\u2005?)"
    ,"ug");
    req.params.q = req.params.q.replace(cjk, "$1\u2005");
    // END CJK spacing

    return mathoid.handleRequest(res,
        req.params.q,
        req.params.type,
        req.params.outformat,
        {},
        req.logger, app.conf,
        app.mjAPI);
});

/**
 * GET /
 * Performs the check get request
 */
router.get('/zlib/:outformat/:type/:q*', (req, res) => {
    if (!(req.params.q)) {
        emitError('q (query) parameter is missing!');
    }
    res.set('Warning',
        '299 mathoid Rendering via GET request discouraged. Consider using the POST endpoint.');
    // if the q parameter contains '/' everything after the first '/' is stored in the req.param[0]
    if (req.params[0]) {
        req.params.q += req.params[0];
    }

    //begin CJK spacing - by libattery
    var cjk = new RegExp(
    "(["
    +"\u1100-\u11FF\u3130-\u318F\uAC00-\uD7AF" //Korean
    +"\u2E80-\u2EFF\u31C0-\u31EF\u3200-\u32FF" //Chinese
    +"\u3400-\u4DBF\u4E00-\u9FBF\uF900-\uFAFF" //Chinese
    +"\u3000-\u303f\u3040-\u30FF\u31F0-\u31FF\uff00-\uff9f" //Japanese
    +"])(\u2005?)"
    ,"ug");
    req.params.q = req.params.q.replace(cjk, "$1\u2005");
    // END CJK spacing

    // base64 encoding also accepts Encoding with URL and Filename Safe Alphabet https://tools.ietf.org/html/rfc4648#section-5
    const buffer = Buffer.from(req.params.q, 'base64');
    return zlib.unzipAsync(buffer, { finishFlush: (zlib.constants || zlib).Z_SYNC_FLUSH })
        .then((q) => mathoid.handleRequest(res,
            q,
            req.params.type,
            req.params.outformat,
            {},
            req.logger, app.conf,
            app.mjAPI));
});
/**
 * POST /
 * Performs the rendering request
 */
router.post('/:outformat?/', (req, res) => {
    let speech = app.conf.speech_on;
    if (req.body.nospeech) {
        speech = false;
    }

    //begin CJK spacing - by libattery
    var cjk = new RegExp(
    "(["
    +"\u1100-\u11FF\u3130-\u318F\uAC00-\uD7AF" //Korean
    +"\u2E80-\u2EFF\u31C0-\u31EF\u3200-\u32FF" //Chinese
    +"\u3400-\u4DBF\u4E00-\u9FBF\uF900-\uFAFF" //Chinese
    +"\u3000-\u303f\u3040-\u30FF\u31F0-\u31FF\uff00-\uff9f" //Japanese
    +"])(\u2005?)"
    ,"ug");
    req.body.q = req.body.q.replace(cjk, "$1\u2005");
    // END CJK spacing

    return mathoid.handleRequest(res,
        req.body.q,
        req.body.type,
        req.params.outformat,
        { speech },
        req.logger,
        app.conf,
        app.mjAPI);

});

module.exports = (appObj) => {

    app = appObj;

    return {
        path: '/',
        skip_domain: true,
        router
    };

};
